# -*- coding: utf-8 -*-
### BEGIN LICENSE
# This file is in the public domain
### END LICENSE

import gtk

from testdrivegtk.helpers import get_builder
import os, string, commands

import gettext
from gettext import gettext as _
gettext.textdomain('testdrivegtk')

class AddOtherTestdrivegtkDialog(gtk.Dialog):
	__gtype_name__ = "AddothertestdrivegtkDialog"

	def __new__(cls, cache):
		"""Special static method that's automatically called by Python when 
		constructing a new instance of this class.

		Returns a fully instantiated AddothertestdrivegtkDialog object.
		"""
		builder = get_builder('AddOtherTestdrivegtkDialog')
		new_object = builder.get_object('addothertestdrivegtk_dialog')
		new_object.finish_initializing(builder, cache)
		return new_object

	def finish_initializing(self, builder, cache):
		"""Called when we're finished initializing.

		finish_initalizing should be called after parsing the ui definition
		and creating a AddothertestdrivegtkDialog object with it in order to
		finish initializing the start of the new AddothertestdrivegtkDialog
		instance.
		"""
		# Get a reference to the builder and set up the signals.
		self.builder = builder
		self.builder.connect_signals(self)
		self.CACHE = cache
		self.PROTO = None
		self.url = None
		self.desc = None
		self.other_isos = []
		# Widgets
		self.txt_other_desc = self.builder.get_object("txt_other_desc")
		self.txt_other_url = self.builder.get_object("txt_other_url")
		#self.txt_other_url.connect("focus-out-event", self.validate_is_iso, self.txt_other_url.get_text())
		self.btn_add_other = self.builder.get_object("btn_add_other")
		self.btn_add_other.connect("clicked", self.on_add_other_iso_clicked)

		self.layout_table = self.builder.get_object("tb_other_iso")

		#Sync Protocol Combo Box
		self.cb_sync_proto = gtk.combo_box_new_text()
		self.cb_sync_proto.append_text('Select Protocol:')
		self.cb_sync_proto.append_text("rsync")
		self.cb_sync_proto.append_text("zsync")
		self.cb_sync_proto.append_text("wget")
		self.cb_sync_proto.append_text("file")
		self.cb_sync_proto.connect('changed', self.on_select_sync_proto)
		self.cb_sync_proto.set_active(0)
		self.layout_table.attach(self.cb_sync_proto, 1,3,2,3, gtk.FILL | gtk.EXPAND, gtk.EXPAND)
		self.cb_sync_proto.show()

	def ok(self, widget, data=None):
		"""The user has elected to save the changes.

		Called before the dialog returns gtk.RESONSE_OK from run().
		"""
		path = "%s/other_isos" % self.CACHE
		try:
			f = open(path,'a')
			for iso in self.other_isos:
				f.write(iso)
			f.close
		except IOError:
			pass
		pass

	def cancel(self, widget, data=None):
		"""The user has elected cancel changes.

		Called before the dialog returns gtk.RESPONSE_CANCEL for run()
		"""
		pass

	def on_add_other_iso_clicked(self, widget):
		add = self.on_validate_iso_url()
		if add is False:
			return

		old_proto = self.url.partition(":")[0]
		self.url.replace(old_proto, self.PROTO)
		iso = "other\tother\t\t%s\t\t%s\n" % (self.url, self.desc)
		self.other_isos.append(iso)
		
		#self.on_info_dlg("ISO added successfully!")

		self.txt_other_desc.set_text("")
		self.txt_other_url.set_text("")
		self.cb_sync_proto.set_active(0)

	def on_select_sync_proto(self, widget):
		model = widget.get_model()
		index = widget.get_active()
		if index:
			self.PROTO = model[index][0]
		else:
			self.PROTO = None

	def on_error_dlg(self, data=None):
		errorbox = gtk.MessageDialog(self, 
			gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR, 
			gtk.BUTTONS_CLOSE, data)
		response = errorbox.run()
		errorbox.destroy()

	def on_info_dlg(self, data=None):
		errorbox = gtk.MessageDialog(self, 
			gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_INFO, 
			gtk.BUTTONS_CLOSE, data)
		errorbox.run()
		errorbox.destroy()

	def on_validate_iso_url(self):
		file = self.txt_other_url.get_text().strip()
		desc = self.txt_other_desc.get_text().replace(' ','-')

		if len(desc) == 0:
			self.on_error_dlg("Please insert a Description")
			self.set_focus(self.txt_other_url)
			return False

		if len(file) == 0:
			self.on_error_dlg("Please insert a URL")
			self.set_focus(self.txt_other_url)
			return False

		self.desc = desc
		# If it's a URL, assume it's good
		for i in ("http", "ftp", "rsync", "file", "zsync"):
			if string.find(file, "%s://" % i) == 0:
				self.url = file
		#If it's a local path, test it for viability
		if commands.getstatusoutput("file \"%s\" | grep -qs \"ISO 9660\"" % file)[0] == 0:
			#return("file://%s" % file)
			self.url ="file://%s" % file

		if self.url is None:
			self.on_error_dlg("Invalid ISO URL [%s]" % file)
			return False

		# Validate if url exists
		proto = self.url.partition(":")[0]
		url = self.url
		print proto
		if proto == 'rsync' or proto == 'zsync':
			url = url.replace(proto, 'http')
		if proto == 'file':
			pass
		elif os.system("wget --spider -S %s 2>&1 | grep 'HTTP/1.. 200 OK'" % url) != 0:
			self.on_error_dlg("ISO not found at [%s]" % url)
			return False	

		if self.PROTO is None:
			self.on_error_dlg("No sync protocol has been selected, please select one.")
			#self.set_focus(self.cb_sync_proto)
			return False
		else:
			return True

if __name__ == "__main__":
	dialog = AddOtherTestdrivegtkDialog()
	dialog.show()
	gtk.main()
